/*
 * Copyright 1997-2003 Samuel Audet <guardia@step.polymtl.ca>
 *                     Taneli Lepp <rosmo@sektori.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. The name of the author may not be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define INCL_DOS
#include <os2.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <memory.h>

#include "wav.h"

inline int min( int x, int y )
{
	return (x < y) ? x : y;
}


WAV::WAV()
{
	m_pInfo = 0;
	m_pWavPool = 0;
}

WAV::~WAV()
{
	close();
}

int WAV::open(char *filename, int &samplerate,
              int &channels, int &bits, int &format)
{
	m_nPos = 0;
	m_nRemain = 0;

	int	nRetVal = NO_WAV_FILE;

	m_pInfo = (tta_info*)malloc( sizeof(tta_info) );
	if(m_pInfo == 0) {
		return nRetVal;
	}

	if(open_tta_file( filename, m_pInfo, 0 ) >= 0) {
		if (player_init( m_pInfo ) >= 0) {
			samplerate = m_pInfo->SAMPLERATE;
			channels = m_pInfo->NCH;
			bits = m_pInfo->BPS;
			format = WAVE_FORMAT_PCM;

			if(m_pWavPool) {
				delete [] m_pWavPool;
			}
			m_pWavPool = new unsigned char [PCM_BUFFER_LENGTH  * m_pInfo->BSIZE * m_pInfo->NCH];
			if(m_pWavPool) {
				nRetVal = 0;
			}
		}
	}

	if(nRetVal != 0) {
		close_tta_file ( m_pInfo );
		free( m_pInfo );
		m_pInfo = 0;
	}

	return nRetVal;
}

int WAV::close()
{
	if(m_pInfo) {
		player_stop();
		close_tta_file ( m_pInfo );
		free( m_pInfo );
		m_pInfo = 0;
	}

	if(m_pWavPool) {
		delete [] m_pWavPool;
		m_pWavPool = 0;
	}
	
	return 0;
}

int WAV::readData(char *buffer, int bytes)
{
	int total = 0;
	while(bytes > 0) {
		if(m_nRemain) {
			register int n = min( bytes, m_nRemain );
			memcpy( buffer, &m_pWavPool[m_nPos], n );

			m_nRemain -= n;
			m_nPos    += n;
			bytes     -= n;
			buffer    += n;
			total     += n;
		} else {
			int nRetVal = get_samples( m_pWavPool );
			if(nRetVal <= 0) {
				break;
			}
			m_nRemain = nRetVal * m_pInfo->NCH * m_pInfo->BSIZE;
			m_nPos = 0;
		}
	}
	return total;
}

int WAV::filepos()
{
	if(m_pInfo) {
		int ms = get_position() * SEEK_STEP;
		if(ms > m_pInfo->LENGTH * 1000) {
			ms = m_pInfo->LENGTH * 1000;
		}
		return ms;
	}
	return 0;
}

int WAV::filelength()
{
	if(m_pInfo) {
		return m_pInfo->LENGTH * 1000;
	}
	return 0;
}

int WAV::jumpto(long offset)
{
	if(m_pInfo) {
		if(set_position( offset / SEEK_STEP ) == 0) {
			m_nPos = 0;
			m_nRemain = 0;
		}
	}

	return 0;
}

